@@ -8,7 +8,7 @@ import sys
 import weakref
 from abc import ABC, abstractmethod
 from contextlib import suppress
-from typing import ClassVar
+from typing import Any, ClassVar
 
 import dask
 from dask.utils import parse_timedelta
@@ -214,7 +214,9 @@ class Listener(ABC):
 
         return _().__await__()
 
-    async def on_connection(self, comm: Comm, handshake_overrides=None):
+    async def on_connection(
+        self, comm: Comm, handshake_overrides: dict[str, Any] | None = None
+    ) -> None:
         local_info = {**comm.handshake_info(), **(handshake_overrides or {})}
 
         timeout = dask.config.get("distributed.comm.timeouts.connect")
